﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Data.SqlClient;
using System.Xml;
using System.Xml.Serialization;
using System.Collections.ObjectModel;
using System.Reflection;
using System.Collections;

namespace EntityMap.Meta
{
    /// <summary>
    /// This class is used for sql statement that return collection of data.
    /// </summary>
    public class SelectStatement : SqlStatement
    {
        [XmlAttribute( "resultMap" )]
        public string ResultMap { get;set; }

        public SelectStatement() {}

        public override object Process<T>( Context ctx )
        {
            IList<T> entities = new Collection<T>();

            Type entityType = typeof( T );

            using( SqlConnection conn = new SqlConnection( ctx.Mapper.ConnectionString ) )
            {
                SqlCommand cmd = new SqlCommand( this.Command, conn );

                if( ctx.ParameterMap != null )
                {
                    cmd.Parameters.AddRange( ctx.ParameterMap.GetSqlParameters( ctx.Parameter ) );
                }

                conn.Open();

                SqlDataReader reader = cmd.ExecuteReader();

                // create and fill new object
                while( reader.Read() )
                {
                    T obj = Activator.CreateInstance<T>();

                    foreach( Field fi in ctx.ResultMap.Fields )
                    {
                        PropertyInfo pinfo = entityType.GetProperty( fi.Property );

                        object result;

                        // check for join
                        if( fi is ComplexField )
                        {
                            MethodInfo minfo = typeof( EntityMapper ).GetMethod( "ExecuteStatement" );
                            MethodInfo gminfo = minfo.MakeGenericMethod( pinfo.PropertyType );

                            result = gminfo.Invoke( ctx.Mapper,
                                new object[] { ( (ComplexField)fi ).SelectStatement,
                                ctx.Parameter } );
                        }
                        else
                        {
                            result = reader[fi.Column];
                        }
                        pinfo.SetValue( obj, result, null );
                    }
                    entities.Add( obj );
                }
            }
            return (IList)entities;
        }
    }
}
